﻿/*
 * fbasync - An Asynchronous Facebook API for .NET
 * Copyright (C) 2007 ittybittysoft
 * Email: info@ittybittysoft.com
 * Portions of the library are based on the Facebook Developer Toolkit
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Xml;

namespace fbasync
{
    [DataContract(), Serializable()]
    public class Group
        : FacebookEntity
    {
        public readonly static IEnumerable<string> AllFields =
            new string[] { "gid", "name", "nid", "pic", "pic_big", "pic_small", "description",
                           "group_type", "group_subtype", "recent_news", "creator", "update_time",
                           "office", "website", "venue" };

        public const string DefaultOrderBy = "name";
        public const string FacebookEntity = "group";


        [DataMember]
        public long? gid { get; set; }

        [DataMember]
        public string name { get; set; }

        [DataMember]
        public long? nid { get; set; }

        [DataMember]
        public string pic { get; set; }

        [DataMember]
        public string pic_big { get; set; }

        [DataMember]
        public string pic_small { get; set; }

        [DataMember]
        public string description { get; set; }

        [DataMember]
        public string group_type { get; set; }

        [DataMember]
        public string group_subtype { get; set; }

        [DataMember]
        public string recent_news { get; set; }

        [DataMember]
        public long? creator { get; set; }

        [DataMember]
        public DateTime? update_time { get; set; }

        [DataMember]
        public string office { get; set; }

        [DataMember]
        public string website { get; set; }

        [DataMember]
        public Venue venue { get; set; }

        public static IList<Group> FromFqlResult(XmlDocument fqlResult, bool keepXml)
        {
            List<Group> results = fqlResult.GetFqlResults(keepXml, e =>
                new Group
                {
                    creator = e.GetChildLong("eid"),
                    description = e.GetChildText("description"),
                    gid = e.GetChildLong("gid"),
                    group_subtype = e.GetChildText("group_subtype"),
                    group_type = e.GetChildText("group_type"),
                    name = e.GetChildText("name"),
                    office = e.GetChildText("office"),
                    nid = e.GetChildLong("nid"),
                    pic = e.GetChildText("pic"),
                    pic_big = e.GetChildText("pic_big"),
                    pic_small = e.GetChildText("pic_small"),
                    recent_news = e.GetChildText("recent_news"),
                    update_time = e.GetChildUnixTime("update_time"),
                    venue = new Venue(e["venue", FacebookConnection.FacebookNamespace]),
                    website = e.GetChildText("website")
                });
            return results;
        }
    }
}
